'''
classifier.py

Sort data from a user's visits.
Calculate results: precision and recall.

'''

from visits_tracker.models import VisitFeatures
from sklearn import tree


def sorter():
    '''
    Predict the value of the place type of the next visit.

    '''

    #Load data of visits
    visitFeaturesAllUsers = VisitFeatures.objects.all()

    #List users
    users = []
    for vf in visitFeaturesAllUsers:
        user = vf.visit.user_id
        if(not user in users):
            users.append(user)

    #Select training set
    resultsAllUsers = []

    for user in users:
        user_results = {}
        
        visitFeatures = [vf for vf in visitFeaturesAllUsers if vf.visit.user_id == float(user)]

        length = len(visitFeatures)
        if(length > 100):
            training = 80
            if(length>200):
                training = 180
                if(length > 300):
                    training = 290
        else:
            training = length/2

        total = 0.
        correct = 0.
        truePositive = [0., 0., 0.]
        falsePositive = [0., 0., 0.]
        falseNegative = [0., 0., 0.]
        trueNegative = [0., 0., 0.]
        totalFail = 0
        
        first = True

        #Classify data
        for j in range(training,length-10):

            X = []
            VF = []

            total += 1
            for k in range(j-training,j+1):
                VF.append(visitFeatures[k])

            #vf_dic = {'0': ['1','0','0'], '1':['0','1','0'], '2':['0','0','1']}
            #Training set
            vf_dic = {'0': ['0','0'], '1':['1','0'], '2':['0','1']}
            for vf in VF:
                placeType = str(vf.placeType)
                previosPlaceType = str(vf.previous_placeType)
                X.append([ 
                        vf_dic[str(placeType)][0],
                        vf_dic[str(placeType)][1],
                        str(vf.timeSlot),
                        vf.workingDay,
                        str(vf.day_of_week),
                        str(vf.hour_of_day),
                        str(vf.half_of_day),
                        str(vf.half_of_month),
                        str(vf.third_of_day),
                        str(vf.week_of_month),
                        placeType,
                        int(vf.accuracy),
                        int(vf.distanceToHome),
                        previosPlaceType,
                        vf_dic[previosPlaceType][0],
                        vf_dic[previosPlaceType][1],
                        str(vf.previous_timeSlot),
                        int(vf.duration),
                        int(vf.averageTimePlaceVisit)
                        ])

            #Decision tree
            dt = tree.DecisionTreeClassifier() 

            #Predict place type home and work values
            pred = [-1, -1]
            index = -1
            for feature in [0,1]:

                index += 1
                #Training set
                train = X[:len(X)-2]

                #Set the class labels for the training samples
                y = []
                for feat in X[1:len(X)-1]:
                    y.append(feat[feature])

                #Training tree
                dt = dt.fit(train,y)

                #Predict value
                pred[index] = dt.predict([X[len(X)-2]])

            #Compare prediction values of both decision trees
            prediction = '-1'
            if((pred[0] == '0') and (pred[1] == '0')):
                prediction = '0'
            elif((pred[0] == '1') and (pred[1] == '1')):
                prediction = '-1'
                totalFail += 1
                total -= 1
            elif((pred[0] == '1') and (pred[1] == '0')):
                prediction = '1'
            elif((pred[0] == '0') and (pred[1] == '1')):
                prediction = '2'
            else:
                prediction = '-2'

            #Results
            if(prediction in [ '0', '1', '2']):
                if(prediction == placeType):
                    correct += 1
                    truePositive[int(prediction)] += 1
                else:
                    falsePositive[int(prediction)] += 1
                    falseNegative[int(placeType)] += 1

            if(first):
                if(prediction == "-1"):
                    total += 1
            if(first):
                if(prediction == "-1"):
                    total -= 1
                else:
                    first = False

        #Save results
        user_results['user'] = user
        user_results['training'] = training
        user_results['correct'] = correct
        user_results['total'] = total
        user_results['truePositive'] = truePositive
        user_results['falsePositive'] = falsePositive
        user_results['falseNegative'] = falseNegative
        user_results['totalFail'] = totalFail

        #Calculate precision and recall result values
        precision = []
        recall = []
        for k in range(2):
            if((truePositive[k] + falsePositive[k]) > 0):
                precision.append(truePositive[k]/(truePositive[k] + falsePositive[k]))
            else:
                precision.append(0)

            if((truePositive[k] + falseNegative[k])>0):
                recall.append(truePositive[k]/(truePositive[k] + falseNegative[k]))
            else:
                recall.append(0)
        user_results['precision'] = precision
        user_results['recall'] = recall

        resultsAllUsers.append(user_results)

    #Return results
    return resultsAllUsers