#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
import_data.py

Import data from csv file and save in the database.

'''

import csv
from .models import Visit, VisitFeatures
from .serializers import VisitSerializer, VisitFeaturesSerializer
from util import extract_features_visit, compute_vincenty_distance
from .clustering import aglomerativeClust


def updatePlaceType():
    '''
    Update data of all Visit Features.

    Gruop data with clustering.
    Save new data in database.

    '''

    #Load data of visits
    visitFeaturesAllUsers = VisitFeatures.objects.all()

    #List users
    users = []
    for vf in visitFeaturesAllUsers:
        user = vf.visit.user_id
        if(not user in users):
            users.append(user)

    #Update data user
    for user in users:

        #Load a user's data
        visitFeatures = [vf for vf in visitFeaturesAllUsers if vf.visit.user_id == float(user)]

        place = [0,0,0]
        minutes = [0,0,0]
        minDuration = [100,100,100]
        maxDuration = [0,0,0]

        home = None

        #Clustering
        labels = aglomerativeClust(visitFeatures)

        clusters = [0. for o in range(max(labels)+1)]

        for n in range(len(visitFeatures)):
            visitFeatures[n].placeType = str(labels[n])
            clusters[int(labels[n])] += 1

        cardinal = [k for k in clusters]
        max1 = max(cardinal)
        cardinal.remove(max1)
        max2 = max(cardinal)
        placeTypeHome, placeTypeWork = -1, -1


        for k in range(len(clusters)):

            if((placeTypeHome == -1) and (max1 == clusters[k])):
                placeTypeHome = k

            if((placeTypeWork == -1) and (max2 == clusters[k])):
                placeTypeWork = k

        #Update place type values
        for vf in visitFeatures:

            if(str(vf.placeType) == str(placeTypeHome)):
                vf.placeType = '1'
                if home == None:
                    home = vf.visit

            elif(str(vf.placeType) == str(placeTypeWork)):
                vf.placeType = '2'

            else:
                vf.placeType = '0'

        #Calculate average place time visit
        for vf in visitFeatures:

            if(vf.placeType == '0'):
                place[0] += 1
                minutes[0] += vf.duration
                if(vf.duration > maxDuration[0]):
                    maxDuration[0] = vf.duration
                if(vf.duration < minDuration[0]):
                    minDuration[0] = vf.duration

            elif(vf.placeType == '1'):
                place[1] += 1
                minutes[1] += vf.duration
                if(vf.duration > maxDuration[1]):
                    maxDuration[1] = vf.duration
                if(vf.duration < minDuration[1]):
                    minDuration[1] = vf.duration

            elif(vf.placeType == '2'):
                place[2] += 1
                minutes[2] += vf.duration
                if(vf.duration > maxDuration[2]):
                    maxDuration[2] = vf.duration
                if(vf.duration < minDuration[2]):
                    minDuration[2] = vf.duration

        #Update average place time visit
        for i in range(3):
            if(place[i] > 0):
                minutes[i] /= place[i]
            else:
                minutes[i] = 0

        #Update previos visit information
        previous_placeType = "0"
        previous_timeSlot = 0

        #Update data to database
        for vf in visitFeatures:
            validated_data = {}
            validated_data['visit'] = vf.visit
            validated_data['placeType']=vf.placeType
            validated_data['timeSlot']=vf.timeSlot
            validated_data['workingDay']=vf.workingDay
            validated_data['day_of_week']=vf.day_of_week
            validated_data['hour_of_day']=vf.hour_of_day
            validated_data['half_of_day']=vf.half_of_day
            validated_data['duration']=vf.duration
            validated_data['month']=vf.month
            validated_data['half_of_month']=vf.half_of_month
            validated_data['third_of_day']=vf.third_of_day
            validated_data['week_of_month']=vf.week_of_month
            validated_data['accuracy']=vf.accuracy
            validated_data['distanceToHome'] = compute_vincenty_distance(vf.visit, home)
            validated_data['previous_placeType'] = previous_placeType
            validated_data['previous_timeSlot'] = previous_timeSlot
            validated_data['averageTimePlaceVisit'] = minutes[int(vf.placeType)]
            validated_data['minTimePlaceVisit'] = minDuration[int(vf.placeType)]
            validated_data['maxTimePlaceVisit'] = maxDuration[int(vf.placeType)]
            constructor = VisitFeaturesSerializer()
            constructor.update(validated_data)
            previous_placeType = vf.placeType
            previous_timeSlot = vf.timeSlot
            

def addData(csv_file = '', delimeter_csv = ','):
    '''
    Save data of visits

    Load data from a csv file.
    Save visits data.
    Calculate visit features values and update database.

    '''

    #Load csv file
    with open(csv_file, 'rb') as f:
        reader = csv.reader(f, delimiter=delimeter_csv)
        dataSet = list(reader)

    #Remove header
    dataSet.pop(0) 

    #Load visits data
    for data in dataSet:
        if len(data) == 9:

            #Visit data
            validated_data = {}
            validated_data['latitude']=float(data[1])
            validated_data['longitude'] = float(data[2])
            validated_data['horizontalAccuracy']=float(data[3])
            validated_data['arrivalDate']=float(data[4])
            validated_data['departureDate']=float(data[5])
            validated_data['receivedDate']=float(data[6])
            validated_data['user_id']=float(data[7])
            constructor = VisitSerializer()
            visit = constructor.create(validated_data=validated_data)

            #Visits features data
            features = extract_features_visit(visit)
            validated_data = {}
            validated_data['visit'] = features.visit
            validated_data['placeType']=features.placeType
            validated_data['timeSlot']=features.timeSlot
            validated_data['workingDay']=features.workingDay
            validated_data['day_of_week']=features.day_of_week
            validated_data['hour_of_day']=features.hour_of_day
            validated_data['half_of_day']=features.half_of_day
            validated_data['duration']=features.duration
            validated_data['month']=features.month
            validated_data['half_of_month']=features.half_of_month
            validated_data['third_of_day']=features.third_of_day
            validated_data['week_of_month']=features.week_of_month
            validated_data['accuracy']=features.accuracy
            validated_data['distanceToHome'] = 0
            validated_data['previous_placeType'] = ""
            validated_data['previous_timeSlot'] = ""
            constructor = VisitFeaturesSerializer()
            constructor.create(validated_data)

