#!/usr/bin/python
# coding=utf-8

'''
serializers.py

Django serializers.

Visit and VisitFeatures serializers.
Create data rows in MySQL database.

'''

import util
from .models import Visit, VisitFeatures
from decimal import *


class VisitSerializer():
    '''
    Visit serializer

    '''

    def create(self, validated_data):

        #Discard visit if useless information
        if (validated_data['departureDate'] == 64092211200 and validated_data['arrivalDate'] == -62135769600):#useless information
            return None

        user = validated_data['user_id']
        
        #If exists (visit's arrival notification) update data
        sameVisits = Visit.objects.filter(user_id = user,
                            arrivalDate = validated_data['arrivalDate'])
        #to detect multiple records
        if len(sameVisits)>0:
            visit = sameVisits[len(sameVisits)-1]

            #Save the visit with better data
            if (float(visit.horizontalAccuracy) > float(validated_data['horizontalAccuracy'])):
                visit.latitude = validated_data['latitude']
                visit.longitude = validated_data['longitude']
                visit.horizontalAccuracy = validated_data['horizontalAccuracy']

            if (visit.departureDate != validated_data['departureDate']):
                visit.departureDate = validated_data['departureDate']
                visit.receivedDate_exit = validated_data['receivedDate']

            visit.save()
            return visit

        else:

            #If not, create a new row with the visit information
            visit = Visit(user_id = user,
                latitude = validated_data['latitude'],
                longitude = validated_data['longitude'],
                horizontalAccuracy = validated_data['horizontalAccuracy'],
                arrivalDate = validated_data['arrivalDate'],
                departureDate = validated_data['departureDate'])

            visit.save()

            return visit

        return visit


class VisitFeaturesSerializer():
    '''
    VisitFeatures serializer

    '''

    def create(self, validated_data):
        visitFeatures = None

        
        sameVisitFeatures = VisitFeatures.objects.filter(visit = validated_data['visit'])

        if len(sameVisitFeatures)>0:
            #If exists, no work
            print "This information is already stored" 

        else:
            #If not, create a new row with the visit features information
            visitFeatures = VisitFeatures.objects.create(visit = validated_data['visit'],
                                                        placeType = validated_data['placeType'],
                                                        accuracy = validated_data['accuracy'],
                                                        timeSlot = validated_data['timeSlot'],
                                                        workingDay = validated_data['workingDay'],
                                                        day_of_week = validated_data['day_of_week'],
                                                        hour_of_day = validated_data['hour_of_day'],
                                                        half_of_day = validated_data['half_of_day'],
                                                        duration = validated_data['duration'],
                                                        month = validated_data['month'],
                                                        half_of_month = validated_data['half_of_month'],
                                                        third_of_day = validated_data['third_of_day'],
                                                        week_of_month = validated_data['week_of_month']
                                                        )

    def update(self, validated_data):
        '''
        Update data of a VisitFeatures
        
        '''

        sameVisitFeatures = VisitFeatures.objects.filter(visit = validated_data['visit'])

        if len(sameVisitFeatures)>0:
            #If exists, update data
            visitFeatures = sameVisitFeatures[0]
            visitFeatures.placeType = validated_data['placeType']
            visitFeatures.distanceToHome = distanceToHome = validated_data['distanceToHome']
            visitFeatures.previous_placeType = previous_placeType = validated_data['previous_placeType']
            visitFeatures.previous_timeSlot = previous_timeSlot = validated_data['previous_timeSlot']
            visitFeatures.save()

        else:
            #If not, create a new row with the visit features information
            visitFeatures = VisitFeatures.objects.create(visit = validated_data['visit'],
                                                        placeType = validated_data['placeType'],
                                                        accuracy = validated_data['accuracy'],
                                                        distanceToHome = validated_data['distanceToHome'],
                                                        previous_placeType = validated_data['previous_placeType'],
                                                        previous_timeSlot = validated_data['previous_timeSlot'],
                                                        timeSlot = validated_data['timeSlot'],
                                                        workingDay = validated_data['workingDay'],
                                                        day_of_week = validated_data['day_of_week'],
                                                        hour_of_day = validated_data['hour_of_day'],
                                                        half_of_day = validated_data['half_of_day'],
                                                        duration = validated_data['duration'],
                                                        month = validated_data['month'],
                                                        half_of_month = validated_data['half_of_month'],
                                                        third_of_day = validated_data['third_of_day'],
                                                        week_of_month = validated_data['week_of_month']
                                                        )
            visitFeatures.save()
