#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
util.py

Set of methods used in the prediction process.

Extrac features from visit information.
Calculate distance of two gps locations.
Reset MySQL database.

'''

import datetime

from vincenty import vincenty
from .models import Visit, VisitFeatures
from timezonefinder import TimezoneFinder
from pytz import timezone, utc
from pytz.exceptions import UnknownTimeZoneError


def extract_features_visit(visit):
    '''
    Extract features from a visit and returns the visit features.

    '''

    visitProcessed = VisitFeatures(visit = visit)

    #Calculate earth time zone
    tf = TimezoneFinder()
    timezone_name = tf.timezone_at(lng=visit.longitude, lat=visit.latitude)

    try:
        tz = timezone(timezone_name)

        if visit.arrivalDate != -62135769600:
            arr = True
            arrival = datetime.datetime.utcfromtimestamp(visit.arrivalDate)
        else:
            arrival = datetime.datetime.utcfromtimestamp(visit.departureDate)
        aware_datetime_arrival = arrival.replace(tzinfo=tz)
        aware_datetime_in_utc_arrival = aware_datetime_arrival.astimezone(utc)
        tz_arrival = tz.localize(arrival)
        tt_arrival = tz_arrival.timetuple()

        #Calculate features
        visitProcessed.workingDay = tt_arrival.tm_wday < 5
        visitProcessed.timeSlot = tt_arrival.tm_hour % 24 
        visitProcessed.day_of_week = arrival.weekday() + 1
        visitProcessed.hour_of_day = arrival.hour
        visitProcessed.half_of_day = 1 if arrival.hour < 12 else 2
        visitProcessed.third_of_day = 2 if (7 < arrival.hour < 15) else (3 if 14 < arrival.hour < 23 else 1)
        visitProcessed.month = arrival.month
        visitProcessed.half_of_month = 1 if arrival.day < 16 else 2
        week = arrival.day / 7
        visitProcessed.week_of_month = week + 1 if week < 4 else 4
        visitProcessed.accuracy = visit.horizontalAccuracy
        visitProcessed.placeType = ""
        

    except UnknownTimeZoneError:
        print "UnknownTimeZoneError: ", visitRes
        return None

    return visitProcessed


def compute_vincenty_distance(visit_ori, visit_des):
    '''
    Computes trip distance in metres by using vincenty distance.
    '''

    vincenty_distance = vincenty((float(visit_ori.latitude), float(visit_ori.longitude)), (float(visit_des.latitude), float(visit_des.longitude))) * 1000.#in metres
    return vincenty_distance


def resetDataBase():
    '''
    Reset MySQL database
    '''

    VisitFeatures.objects.all().delete()
    Visit.objects.all().delete()


